import IKTransaction from './IKTransaction';
import IKResponse, { IKBadRequestResponse, IKSuccessResponse } from './API/IKResponse';
import IKGlobals from './IKGlobals';
//
export default class IKEventProcessor {
  constructor(event, context, config) {
    this.event = event;
    this.context = context;
    this.apiConfig = config;
  }
  async processEvent(execution) {
    const resp = await _processRawEvent(execution);
    if (resp.statusCode != 200) throw new Error(JSON.stringify(resp.body));
    else return resp;
  }
  async _processRawEvent(execution) {
    if (this.event.Records && this.event.Records.length > 0) { //safe check for empty events?
      //init transaction for all records
      await (new IKTransaction(this.event, this.context, this.apiConfig)).execute(async (transaction) => {
        let answered = false;
        //for each available event
        for (let eventRecord of this.event.Records) {
          //Get event content
          const parsedEventRecord = JSON.parse(eventRecord.body);
          //Call execution
          let resp = await execution(transaction, parsedEventRecord);
          if (resp.getCode() != 200) { //if execution failed, respond and mark as answered
            answered = true;
            return resp;
          }
          if (answered) break; //stop loop if answer
        }
        if (!answered) return IKSuccessResponse(); //answer success if not answered
      });
    } else return IKBadRequestResponse(IKGlobals.ErrorResponseNoRecords, IKGlobals.ErrorCode_NoRecords); //no event to be processed?
  }
}
